"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var UserModel_1 = require("./UserModel");
var angular = require("angular");
var $ = require("jquery");
var bcrypt = require("bcryptjs");
var User = (function (_super) {
    __extends(User, _super);
    function User(user, userFactory) {
        var _this = _super.call(this) || this;
        _this.userFactory = userFactory;
        _this.passwordHashed = false;
        if (!user)
            return _this;
        var keys = Object.keys(_this);
        for (var i = 0; i < keys.length; ++i) {
            if (user[keys[i]] !== undefined && (typeof user[keys[i]]) !== 'function')
                _this[keys[i]] = user[keys[i]];
        }
        _this.hashPassword();
        return _this;
    }
    User.prototype.LoginModel = function () {
        return { username: this.username, password: this.password };
    };
    ;
    User.prototype.hashPassword = function () {
        if (this.passwordHashed || !this.password || this.password === '') {
            this.passwordHashed = true;
            return this.password;
        }
        this.password = this.userFactory.HashString(this.password);
        this.passwordHashed = true;
    };
    User.prototype.toJson = function () {
        return JSON.stringify(this);
    };
    User.prototype.getBland = function () {
        return {
            _id: this._id,
            username: this.username,
            password: this.password,
            first_name: this.first_name,
            last_name: this.last_name,
            email: this.email,
            role: this.role
        };
    };
    return User;
}(UserModel_1.UserModel));
exports.User = User;
var UserFactory = (function () {
    function UserFactory($http, $rootScope) {
        this.$http = $http;
        this.$rootScope = $rootScope;
        this.LoggedIn = false;
        this.LoadCredentials();
        UserFactory.initialized = true;
    }
    UserFactory.prototype.Initialize = function () {
        if (!UserFactory.initialized || !this.user)
            this.LoadCredentials();
        this.updateRootScope();
    };
    UserFactory.prototype.getSelectList = function (resolve, reject) {
        this.httpServerCall('/api/user/', 'get', undefined, function (data) {
            var list = data.data;
            var resultList = [];
            for (var i = 0; i < list.length; i++) {
                resultList.push(new Object({ label: list[i].first_name, value: list[i]._id }));
            }
            resolve(resultList);
        }, function (error) {
            reject(error);
        });
    };
    UserFactory.prototype.getOneById = function (id, resolve, reject) {
        this.httpServerCall('/api/user/' + id, 'get', undefined, function (data) {
            resolve(data.data);
        }, function (error) {
            reject(error);
        });
    };
    UserFactory.prototype.StoreCredentials = function (user, token) {
        if (!user || !token)
            return;
        this.user = user;
        this.token = token;
        this.LoggedIn = true;
        user.password = '';
        if (user['userFactory'] != undefined)
            user['userFactory'] = undefined;
        localStorage.setItem('user', angular.toJson(user));
        localStorage.setItem('token', token);
        this.updateRootScope();
    };
    UserFactory.prototype.ClearCredentials = function () {
        this.user = undefined;
        this.token = undefined;
        this.LoggedIn = false;
        localStorage.removeItem('token');
        localStorage.removeItem('user');
        this.updateRootScope();
    };
    UserFactory.prototype.updateRootScope = function () {
        if (this.user && this.user.role)
            this.$rootScope.userRole = this.user.role;
        else
            this.$rootScope.userRole = 'Guest';
        this.$rootScope.loggedIn = this.LoggedIn;
    };
    UserFactory.prototype.LoadCredentials = function () {
        var token = localStorage.getItem('token');
        var user = undefined;
        if (localStorage.getItem('user') !== undefined)
            user = angular.fromJson(localStorage.getItem('user'));
        if (!token || !user || !user.username) {
            this.LoggedIn = false;
            this.updateRootScope();
            return;
        }
        this.token = token;
        this.setUser(user);
        this.LoggedIn = true;
        this.updateRootScope();
    };
    UserFactory.prototype.setUser = function (user) {
        this.user = user;
        this.updateRootScope();
        return user;
    };
    UserFactory.prototype.httpServerCall = function (url, method, data, success, failure) {
        if (failure === void 0) { failure = undefined; }
        url = url.replace('//', '/');
        var headers = { 'Content-Type': 'application/x-www-form-urlencoded' };
        if (this.token) {
            headers['Authorization'] = 'Bearer ' + this.token;
        }
        var options = {
            url: url,
            method: method,
            headers: headers
        };
        if (data) {
            if (options.method.toUpperCase() === 'GET') {
                options['params'] = data;
            }
            else {
                options['data'] = $.param(data);
            }
        }
        this.$http(options).then(function (response) {
            success && success(response);
        }, function (error) {
            failure && failure(error);
        });
    };
    UserFactory.prototype.ChangePassword = function (pswd, newPassword, success, failure) {
        if (failure === void 0) { failure = undefined; }
        this.httpServerCall('/api/user/ChangePassword', 'put', {
            username: this.user.username,
            old_password: pswd,
            password: newPassword
        }, function (response) {
            if (!response || !response.data)
                return failure('Server error.');
            if (response.data.error)
                return failure(response.data.message);
            if (response.data.success)
                return success();
            failure("not a success");
        }, failure);
    };
    UserFactory.prototype.HashString = function (toHash) {
        if (bcrypt === undefined)
            return toHash;
        return bcrypt.hashSync(toHash, UserModel_1.ClientSalt);
    };
    UserFactory.prototype.$get = function () {
        return this;
    };
    UserFactory.Factory = function () {
        return this.$inject;
    };
    return UserFactory;
}());
UserFactory.factoryName = 'UserFactory';
UserFactory.$inject = ['$http', '$rootScope'];
UserFactory.initialized = false;
exports.UserFactory = UserFactory;
UserFactory.$inject.push(function (http, $rootScope) {
    return new UserFactory(http, $rootScope);
});
exports.UserFactoryModule = angular.module('user-factory-module', []);
exports.UserFactoryModule.factory(UserFactory.factoryName, UserFactory.Factory());
//# sourceMappingURL=UserFactory.js.map